from distutils.core import setup

setup(
    name = 'ner_evaluator',
    packages = ['ner_evaluator'],
    version = 'v1.0.1',  # Ideally should be same as your GitHub release tag varsion
    description = "Evaluate Named Entity Recognition (NER) models",
    author="Rennan Valadares",
    author_email="rennanvoa2@gmail.com",
    url = 'https://github.com/rennanvoa2/',
    download_url = 'https://github.com/rennanvoa2/ner_evaluator/archive/refs/tags/V1.0.0.tar.gz',
    keywords = ['ner', 'named entity recognition', 'evaluation','NER evaluation'],
    classifiers = [],
)
